/*
 * Copyright (C) 2012 Charlie Curtsinger
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

#define DEBUG_TYPE "lower_intrinsics"

#include <iostream>

#include "llvm/Module.h"
#include "llvm/Pass.h"
#include "llvm/Instructions.h"

#include "llvm/Support/raw_ostream.h"

#include "IntrinsicLibcalls.h"

using namespace std;
using namespace llvm;

struct LowerIntrinsics: public ModulePass {
	static char ID;
	
	LowerIntrinsics() : ModulePass(ID) {
	}
	
	virtual bool runOnModule(Module &m) {
		InitLibcalls();
		
		for(Module::iterator fun = m.begin(); fun != m.end(); fun++) {
			Function &f = *fun;
			if(f.isIntrinsic() && !isAlwaysInlined(f.getName())) {
				StringRef r = GetLibcall(f.getName());
				
				if(!r.empty()) {
					Function *f_extern = m.getFunction(r);
					if(!f_extern) {
						f_extern = Function::Create(
							f.getFunctionType(),
							Function::ExternalLinkage,
							r,
							&m
						);
					}
					f.replaceAllUsesWith(f_extern);
				} else {
					errs()<<"warning: unable to handle intrinsic "<<f.getName().str()<<"\n";
				}
			}
		}
		
		return true;
	}
};

char LowerIntrinsics::ID = 0;
static RegisterPass<LowerIntrinsics> X("lower-intrinsics", "Replace all intrinsics with direct libcalls");
