/*
 * Copyright (C) 2012 Charlie Curtsinger
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

#include "Global.h"
#include "Function.h"
#include "FunctionLocation.h"
#include "Jump.h"
#include "Util.h"
#include "Heaps.h"

#include <sys/mman.h>

using namespace std;

namespace stabilizer {

	Function::Function(struct fn_info *info, GlobalMapType *globals) : name(info->name), base(info->base), limit(info->limit) {
		void **p = info->refs;
		while(*p != NULL) {
			refs.push_back(*p);
			p++;
		}

		this->globals = globals;

		mprotect(ALIGN_DOWN(getBase(), PAGESIZE), 2*PAGESIZE, PROT_READ | PROT_WRITE | PROT_EXEC);

		current_location = new FunctionLocation(this);
		relocated_count = 0;
	}

	FunctionLocation* Function::relocate() {
		relocated_count++;

		DEBUG("Relocating %s", getName());
		void *new_base = Code_malloc(getTotalSize());
	
		FunctionLocation *new_l = new FunctionLocation(this, new_base);
	
		new(getBase()) Jump(new_base);
	
		if(!current_location->isOriginal()) {
			current_location->decrementUsers();
		}
	
		current_location = new_l;

		DEBUG("Relocated %s: \n      [%p .. %p] -> [%p .. %p]",
			  name,
			  getBase(),
			  (void*)((intptr_t)getBase() + getCodeSize()),
			  new_l->getBase(),
			  (void*)((intptr_t)new_l->getBase() + getCodeSize())
		);
	
		return new_l;
	}
}
