/*
 * Copyright (C) 2012 Charlie Curtsinger
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

#ifndef stabilizer2_Function_h
#define stabilizer2_Function_h

#include <stdio.h>
#include <sys/mman.h>
#include <stdint.h>

#include "Metadata.h"
#include "Global.h"
#include "Heaps.h"

//#include "list.h"

#include <vector>

using namespace std;

namespace stabilizer {

	class Function;
	class FunctionLocation;

	//typedef list<void*, MD_malloc, MD_free> PointerListType;
	typedef vector<void*, MDAllocator<void*> > PointerListType;

	//typedef list<Function*, MD_malloc, MD_free> FunctionListType;
	typedef vector<Function*, MDAllocator<Function*> > FunctionListType;

	//typedef list<FunctionLocation*, MD_malloc, MD_free> FunctionLocationListType;
	typedef vector<FunctionLocation*, MDAllocator<FunctionLocation*> > FunctionLocationListType;

	struct fn_info {
		char *name;
		void *base;
		void *limit;
		void **refs;
	};

	struct fn_header {
		uint8_t breakpoint;
		Function *obj;
	};

	class Function : public Metadata {
	private:
		char *name;
		void *base;
		void *limit;
		size_t relocated_count;

		struct fn_header header;

		GlobalMapType *globals;
	
		PointerListType refs;

		FunctionLocation *current_location;

	public:
		Function(struct fn_info *info, GlobalMapType *globals);
		FunctionLocation* relocate();
	
		size_t relocatedCount() {
			return relocated_count;
		}
	
		void placeBreakpoint() {
			struct fn_header *h = (struct fn_header*)base;
			header = *h;
			
			h->breakpoint = 0xCC;
			h->obj = this;
		}
	
		void restoreHeader() {
			struct fn_header *h = (struct fn_header*)base;
			*h = header;
		}

		char* getName() {
			return name;
		}
		
		void* getBase() {
			return base;
		}

		inline GlobalMapType getGlobals() {
			return *globals;
		}

		inline size_t getCodeSize() {
			return (size_t)((uintptr_t)limit - (uintptr_t)base);
		}

		inline size_t getTableSize() {
			return sizeof(void*) * (refs.size() + 2);
		}

		inline size_t getTotalSize() {
			return getCodeSize() + getTableSize();
		}

		inline FunctionLocation* getCurrentLocation() {
			return current_location;
		}

		inline PointerListType::iterator refs_begin() {
			return refs.begin();
		}

		inline PointerListType::iterator refs_end() {
			return refs.end();
		}
	};
}

#endif
