/*
 * Copyright (C) 2012 Charlie Curtsinger
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

#include <string.h>
#include <stdio.h>

#include "Global.h"
#include "Function.h"
#include "FunctionLocation.h"
#include "Util.h"

namespace stabilizer {

	FunctionLocation::FunctionLocation(Function *f) : function(f), base(f->getBase()) {}

	FunctionLocation::FunctionLocation(Function *f, void *b) : function(f), base(b) {
		FunctionLocation *current = function->getCurrentLocation();

		if(current->isOriginal()) {
			void **table = (void**)((intptr_t)base + function->getCodeSize());
			memcpy(base, current->getBase(), function->getCodeSize());

			// Set the users counter to zero
			*((size_t*)table) = 1;
			table++;

			// Set the function location object
			*((FunctionLocation**)table) = this;
			table++;

			GlobalMapType g = f->getGlobals();
			for(PointerListType::iterator r = function->refs_begin(); r != function->refs_end(); r++) {
				void *p = *r;

				if(g.find(p) != g.end()) {
					p = g[p]->getRelocatedBase();
				}

				*table = p;
				table++;
			}
		} else {
			memcpy(base, current->getBase(), function->getTotalSize());
			void **table = (void**)((intptr_t)base + function->getCodeSize());
			*((size_t*)table) = 1;
			table++;
			*((FunctionLocation**)table) = this;
		}
	}

	void FunctionLocation::decrementUsers() {
		void **table = (void**)((intptr_t)base + function->getCodeSize());
		*((size_t*)table) = getUsers()-1;
	}

	size_t FunctionLocation::getUsers() {
		DEBUG("Checking users for %s at %p", function->getName(), getBase());

		size_t *users_ptr = (size_t*)((intptr_t)getBase() + function->getCodeSize());
		return *users_ptr;
	}
	
	bool FunctionLocation::isOriginal() {
		return this->base == function->getBase();
	}
}
